#!/usr/bin/env python
# -*- coding: utf-8 -*-

import wx
from globalStuff import *

# ----------------------------------------------------------------------
# 
class Table():
    def __init__(self, name, cols, pos, imported_from=None):
        self.name=name
        self.cols=cols
        self.pos=pointOnRaster(pos)
        self.imported_from=imported_from
        self.header_color="#00ffff"
        self.size=(0, 0)
        self.size_set=False     # wird in self.calcSize() auf True gesetzt (um Doppel-Berechnung zu blocken)
        self.row_height=None    # wird in self.calcSize() auf die Zeilen-Höhe gesetzt
        self.row_width=None     # wird in self.calcSize() auf die benötigte Zeilen-Breite gesetzt
        self.row_char_len=None
        self.col_y={}           # wird in self.calcSize() auf die y-Position je Spaltenname gesetzt
        for n, t, a in cols:
            self.col_y.update({n: 0})

    # ------------------------------------------------------------------
    # Stellt die Darstellungs-Position der Tabelle auf new_xy ein.
    def moveAbs(self, new_xy):
        self.pos=pointOnRaster(new_xy)

    # ------------------------------------------------------------------
    # Ändert die Darstellungs-Position der Tabelle um delta_xy.
    def moveRel(self, delta_xy):
        dx, dy=delta_xy
        self.pos=(self.pos[0]+dx, self.pos[1]+dy)

    # ------------------------------------------------------------------
    # Liefert True, wenn xy über der Tabelle liegt.
    def isUnderXY(self, xy):
        return self.pos[0]<=xy[0]<=self.pos[0]+self.size[0] and self.pos[1]<=xy[1]<=self.pos[1]+self.size[1]

    # ------------------------------------------------------------------
    # Berechnet anhand der darzustellenden Texte die Abmessungen der
    # Tabelle.
    def calcSize(self, dc):
        if self.size_set:
            return
        self.size_set=True
        dc.SetFont(wx.Font(FONT_SIZE, wx.FONTFAMILY_TELETYPE, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))
        self.row_height=dc.GetTextExtent(self.name)[1]
        self.row_width=[0, 0, 0]
        self.row_char_len=[0, 0, 0]
        for c in self.cols:
            for i in (0, 1, 2):
                self.row_width[i]=max(self.row_width[i], dc.GetTextExtent(c[i])[0])
                self.row_char_len[i]=max(self.row_char_len[i], len(c[i]))
        self.size=(sum(self.row_width)+30, (len(self.cols)+3)*RASTER)
        for i, c in enumerate(self.cols):
            self.col_y[c[0]]=int((i+3)*RASTER)

    # ------------------------------------------------------------------
    # Stellt die Tabelle dar.
    def drawTable(self, dc):
        #dc.SetBrush(wx.Brush("LIGHT BLUE", wx.SOLID))   # http://xoomer.virgilio.it/infinity77/wxPython/Widgets/wx.ColourDatabase.html
        dc.SetBrush(wx.Brush(self.header_color, wx.SOLID))
        dc.DrawRectangle(self.pos[0], self.pos[1], 1+sum(self.row_width)+30, 1+2*RASTER)
        dc.SetBrush(wx.Brush("WHITE", wx.SOLID))
        dc.DrawRectangle(self.pos[0], self.pos[1]+2*RASTER-1, 1+sum(self.row_width)+30, 1+len(self.cols)*RASTER+RASTER)
        dc.SetFont(wx.Font(FONT_SIZE, wx.FONTFAMILY_TELETYPE, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))
        for i, c in enumerate(self.cols):
            y=self.pos[1]+(3+i)*RASTER-self.row_height/2
            dc.DrawText(c[0], self.pos[0]+5,                                           y)
            dc.DrawText(c[1], self.pos[0]+5+self.row_width[0]+10,                      y)
            dc.DrawText(c[2], self.pos[0]+5+self.row_width[0]+10+self.row_width[1]+10, y)
        dc.SetFont(wx.Font(FONT_SIZE, wx.FONTFAMILY_TELETYPE, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD))
        dc.DrawText(self.name, self.pos[0]+5, self.pos[1]+RASTER-self.row_height/2)
        dc.SetFont(wx.Font(FONT_SIZE, wx.FONTFAMILY_TELETYPE, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL))

    # ------------------------------------------------------------------
    # Liefert die TableConnector-Koordinate für die Spalte col_name an
    # der Seite border.
    def getReferencePoint(self, border, col_name):
        if col_name in self.col_y:
            if border=="L":
                return self.pos[0], self.pos[1]+self.col_y[col_name]
            elif border=="R":
                return self.pos[0]+self.size[0], self.pos[1]+self.col_y[col_name]

    # ------------------------------------------------------------------
    # Liefert ein Tupel aus zwei Strings. Der erste String enthält die
    # (statische) Tabellen-Deklarationen, der zweite String enthält die
    # (variablen) Positionsdaten.
    def write(self):
        strg_d="\n"
        if self.imported_from:
            strg_d+="IMPORT_TABLE %s\n"%(self.imported_from,)
            if self.header_color!="#00ffff":
                strg_d+="TABCOLOR %s %s\n"%(self.name, self.header_color)
        else:
            strg_d+="TABLE %s\n"%(self.name,)
            for col in self.cols:
                if col[2]=="":
                    strg_d+="    %s  %s\n"%(col[0].ljust(self.row_char_len[0]), col[1].ljust(self.row_char_len[1]))
                else:
                    strg_d+="    %s  %s  %s\n"%(col[0].ljust(self.row_char_len[0]), col[1].ljust(self.row_char_len[1]), col[2])
            if self.header_color!="#00ffff":
                strg_d+="TABCOLOR %s %s\n"%(self.name, self.header_color)
        strg_p="TABPOS %s (%s,%s)\n"%(self.name, self.pos[0], self.pos[1])
        return strg_d, strg_p
